"use strict";
/*
 * Copyright (c) 2020, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.AliasCommand = exports.Command = void 0;
const command_1 = require("@salesforce/command");
const chalk = require("chalk");
var Command;
(function (Command) {
    Command["List"] = "List";
    Command["Set"] = "Set";
    Command["Unset"] = "Unset";
})(Command = exports.Command || (exports.Command = {}));
class AliasCommand extends command_1.SfdxCommand {
    output(commandName, results) {
        if (results.length === 0) {
            this.ux.log('No results found');
            return;
        }
        this.ux.styledHeader(chalk.blue(`Alias ${commandName}`));
        const column = AliasCommand.tableColumns[commandName];
        this.ux.table(results, column);
        results.forEach((result) => {
            if (result.error) {
                throw result.error;
            }
        });
    }
    async parseArgs() {
        const { argv } = await this.parse({
            flags: this.statics.flags,
            args: this.statics.args,
            strict: this.statics.strict,
        });
        return argv;
    }
}
exports.AliasCommand = AliasCommand;
AliasCommand.tableColumns = {
    [Command.Unset]: { alias: { header: 'Alias' }, success: { header: 'Success' } },
    [Command.Set]: { alias: { header: 'Alias' }, value: { header: 'Value' } },
    [Command.List]: { alias: { header: 'Alias' }, value: { header: 'Value' } },
};
//# sourceMappingURL=alias.js.map