"use strict";
/*
 * Copyright (c) 2020, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
Object.defineProperty(exports, "__esModule", { value: true });
const os = require("os");
const command_1 = require("@salesforce/command");
const core_1 = require("@salesforce/core");
const ts_types_1 = require("@salesforce/ts-types");
const dataCommand_1 = require("../../../../dataCommand");
core_1.Messages.importMessagesDirectory(__dirname);
const messages = core_1.Messages.loadMessages('@salesforce/plugin-data', 'record.get');
const commonMessages = core_1.Messages.loadMessages('@salesforce/plugin-data', 'messages');
class Get extends dataCommand_1.DataCommand {
    async run() {
        this.validateIdXorWhereFlags();
        this.ux.startSpinner('Getting Record');
        const sobject = this.getConnection().sobject(this.flags.sobjecttype);
        try {
            const sObjectId = (this.flags.sobjectid || (await this.query(sobject, this.flags.where)).Id);
            const result = await sobject.retrieve(sObjectId);
            if (!this.flags.json)
                this.logNestedObject(result);
            this.ux.stopSpinner();
            return (0, ts_types_1.toAnyJson)(result);
        }
        catch (err) {
            this.ux.stopSpinner('failed');
            throw new core_1.SfError(err.name, err.message);
        }
    }
}
exports.default = Get;
Get.description = messages.getMessage('description');
Get.examples = messages.getMessage('examples').split(os.EOL);
Get.requiresUsername = true;
Get.flagsConfig = {
    sobjecttype: command_1.flags.string({
        char: 's',
        required: true,
        description: messages.getMessage('sObjectType'),
    }),
    sobjectid: command_1.flags.id({
        char: 'i',
        description: messages.getMessage('sObjectId'),
        exclusive: ['where'],
    }),
    where: command_1.flags.string({
        char: 'w',
        description: messages.getMessage('where'),
        exclusive: ['sobjectid'],
    }),
    usetoolingapi: command_1.flags.boolean({
        char: 't',
        description: messages.getMessage('useToolingApi'),
    }),
    perflog: command_1.flags.boolean({
        description: commonMessages.getMessage('perfLogLevelOption'),
        dependsOn: ['json'],
    }),
};
//# sourceMappingURL=get.js.map